//
//  UITestSuite.swift
//  Copyright © 2020 abc-studio. All rights reserved.
//

import Foundation

protocol UITestSuiteInitializable {
    init()
}

class UITestSuite: UITestSuiteInitializable {
    
    // MARK: - Implementations of Protocol UITestSuiteInitializable
    
    required init() {}
    
    // MARK: - Internal Properties
    
    static var authorizationProvider: UITestAuthorizationProvider?
    
    weak var testRunnerViewController: UITestRunnerViewController! {
        didSet {
            setUp()
        }
    }
    
    var isNewSession = false
    var useLogin = true
    var view: UIView {
        return testRunnerViewController.view
    }
    
    // MARK: - Private Properties
    
    private lazy var strongReferences = [Any]()
    
    // MARK: - Abstract Methods
    
    func setUp() {
        fatalError("setUp has not been implemented")
    }
    
    func runTests() {
        fatalError("runTests has not been implemented")
    }
    
    // MARK: - Internal Methods
    
    func run() {
        guard useLogin else {
            logout()
            runTests()
            return
        }
        guard Self.authorizationProvider?.hasAccessToken == true else {
            signIn()
            return
        }

        if isNewSession || Self.authorizationProvider?.hasRefreshToken == false {
            logout()
            signIn()
        } else {
            Self.authorizationProvider?.refreshToken {
                self.runTests()
            }
        }
    }
    
    func setAsStrongReference(_ any: Any) {
        strongReferences.append(any)
    }
    
    // MARK: - Private Methods
    
    private func signIn() {
        Self.authorizationProvider?.signIn(testRunnerViewController) {
            self.runTests()
        }
    }
    
    private func logout() {
        Self.authorizationProvider?.signOut()
    }
}

extension UITestSuite {
    func present(_ viewController: UIViewController, animated: Bool = false, completion: (() -> Void)? = nil) {
        viewController.modalPresentationStyle = .fullScreen
        testRunnerViewController.present(viewController, animated: animated, completion: completion)
    }

    @discardableResult
    func present(rootViewController: UIViewController, animated: Bool = false, completion: (() -> Void)? = nil) -> UINavigationController {
        let navigationController = UINavigationController(rootViewController: rootViewController)
        navigationController.modalPresentationStyle = .fullScreen
        testRunnerViewController.present(navigationController, animated: animated, completion: completion)
        return navigationController
    }
}